import {
  AddDatabaseRequest,
  DatabaseType,
  databaseNameMapping,
  databaseProtocolMapping,
  useDatabaseAPI,
  useSchemaAPI,
} from '@graphpolaris/shared/lib/data-access';
import React, { useEffect, useRef, useState } from 'react';
import { RequiredInput } from './requiredinput';

export const NewDatabaseForm = (props: { onClose(): void; open: boolean }) => {
  const ref = useRef<HTMLDialogElement>(null);
  const [state, setState] = useState<AddDatabaseRequest>({
    // username: 'root',
    // password: 'DikkeDraak',
    // url: 'https://datastrophe.science.uu.nl/',
    // port: 8529,
    // name: 'Tweede Kamer Dataset',
    // internal_database_name: 'TweedeKamer',
    // type: DatabaseType.ArangoDB,

    // username: 'neo4j',
    // password: 'oL3nNlebrx4le2A0zxaFVqAo3HAvodHxwEiI_7_2JxI',
    // url: '635176c8.databases.neo4j.io',
    // port: 7687,
    // name: 'neo4j',
    // internal_database_name: 'neo4j',
    // type: DatabaseType.Neo4j,

    username: 'neo4j',
    password: 'DevOnlyPass',
    url: 'localhost',
    port: 7687,
    name: 'neo4j',
    protocol: 'neo4j://',
    internal_database_name: 'neo4j',
    type: DatabaseType.Neo4j,
  });
  const api = useDatabaseAPI();
  const schemaApi = useSchemaAPI();
  const [hasError, setHasError] = useState({});

  useEffect(() => {
    if (props.open) ref.current?.showModal();
    else ref.current?.close();
  }, [props.open]);

  useEffect(() => {
    if (!state) return;
    if (state.type === DatabaseType.ArangoDB && state.port === 7687) {
      setState({ ...state, port: 8529 });
    } else if (state.type === DatabaseType.Neo4j && state.port === 8529) {
      setState({ ...state, port: 7687 });
    }
  }, [state.type]);

  /**
   * Validates if the port value is numerical. Only then will the state be updated.
   * @param port The new port value.
   */
  function handlePortChanged(port: string): void {
    if (!isNaN(Number(port))) setState({ ...state, port: Number(port) });
  }

  /** Handles the submit button click. Calls the onSubmit in the props with all the fields. */
  function handleSubmitClicked(): void {
    ref.current?.close();
    if (!Object.values(hasError).some((e) => e === true)) {
      api.AddDatabase(state, { updateDatabaseCache: true, setAsCurrent: true }).then(() => {
        schemaApi.RequestSchema(state.name);
      });
    }
    // props.onSubmit(state);
  }

  return (
    <dialog ref={ref}>
      <form
        className="card flex gap-4 p-5 rounded-sm shadow-lg"
        onSubmit={(event: React.FormEvent) => {
          event.preventDefault();
          handleSubmitClicked();
        }}
      >
        <h1 className="card-title">New Database</h1>
        <RequiredInput
          errorText="This field is required"
          label="Name of the database"
          placeHolder="neo4j"
          value={state.name}
          validate={(v) => {
            setHasError({ ...hasError, name: v.length === 0 });
            return v.length > 0;
          }}
          onChange={(value) =>
            setState({
              ...state,
              name: value,
            })
          }
          type="text"
        />

        <RequiredInput
          errorText="This field is required"
          label="Internal database name"
          placeHolder="internal_database_name"
          value={state.internal_database_name}
          validate={(v) => {
            setHasError({ ...hasError, internal_database_name: v.length === 0 });
            return v.length > 0;
          }}
          onChange={(value) =>
            setState({
              ...state,
              internal_database_name: value,
            })
          }
          type="internalDatabaseName"
        />

        <div className="flex w-full gap-2">
          <div className="w-full">
            <label className="label">
              <span className="label-text">Database Type</span>
            </label>
            <select
              className="select select-bordered w-full max-w-xs"
              value={databaseNameMapping[state.type]}
              onChange={(event) => {
                setState({
                  ...state,
                  type: databaseNameMapping.indexOf(event.currentTarget.value),
                });
              }}
            >
              {databaseNameMapping.map((dbName) => (
                <option value={dbName} key={dbName}>
                  {dbName}
                </option>
              ))}
            </select>
          </div>
          <div className="w-full">
            <label className="label">
              <span className="label-text">Database Protocol</span>
            </label>
            <select
              className="select select-bordered w-full max-w-xs"
              value={state.protocol}
              onChange={(event) => {
                setState({
                  ...state,
                  protocol: event.currentTarget.value,
                });
              }}
            >
              {databaseProtocolMapping.map((protocol) => (
                <option value={protocol} key={protocol}>
                  {protocol}
                </option>
              ))}
            </select>
          </div>
        </div>

        <div className="flex w-full gap-2">
          <RequiredInput
            errorText="This field is required"
            label="Hostname/IP"
            placeHolder="neo4j"
            value={state.url}
            validate={(v) => {
              setHasError({ ...hasError, url: v.length === 0 });
              return v.length > 0;
            }}
            onChange={(value) =>
              setState({
                ...state,
                url: value,
              })
            }
            type="hostname"
          />

          <RequiredInput
            errorText="Must be between 1 and 9999"
            label="Port"
            placeHolder="neo4j"
            value={state.port}
            validate={(v) => {
              setHasError({ ...hasError, port: !(v <= 9999 && v > 0) });
              return v <= 9999 && v > 0;
            }}
            onChange={(value) => handlePortChanged(value)}
            type="port"
          />
        </div>

        <div className="flex w-full gap-2">
          <RequiredInput
            errorText="This field is required"
            label="Username"
            placeHolder="username"
            value={state.username}
            validate={(v) => {
              setHasError({ ...hasError, username: v.length === 0 });
              return v.length > 0;
            }}
            onChange={(value) =>
              setState({
                ...state,
                username: value,
              })
            }
            type="text"
          />
          <RequiredInput
            errorText="This field is required"
            label="Password"
            placeHolder="password"
            value={state.password}
            validate={(v) => {
              setHasError({ ...hasError, password: v.length === 0 });
              return v.length > 0;
            }}
            onChange={(value) =>
              setState({
                ...state,
                password: value,
              })
            }
            type="password"
          />
        </div>

        <div className="card-actions w-full justify-center">
          <button className={`btn btn-primary ${Object.values(hasError).some((e) => e === true) ? 'btn-disabled' : ''}`}>Submit</button>
          <button
            className="btn btn-outline"
            onClick={(e) => {
              e.preventDefault();
              e.stopPropagation();
              props.onClose();
            }}
          >
            Cancel
          </button>
        </div>
      </form>
    </dialog>
  );
};
